"""This is the standard logger used in Kasheesh infrastructure.
"""

import logging


def get_my_logger(name="foo", level=logging.INFO):
    fmt = "[%(levelname)s %(asctime)s]:[%(filename)s:%(lineno)s]-%(funcName)2s(): %(message)s"
    logging.basicConfig(format=fmt, datefmt='%Y-%m-%d %H:%M:%S')
    logger = logging.getLogger(f"logger_{name}")
    logger.setLevel(level)
    return logger





from src.kasheesh_tools.tool_cmd_parser import ToolCmdParser
from collections import defaultdict
import shlex
from datetime import timedelta
import pandas as pd

if __name__ == '__main__':
    testing_map = defaultdict(dict)
    testing_map = {('20210101', '20210131', True, 'day'): {'start_datetime': pd.Timestamp.utcnow().date() - timedelta(days=1), 
                                                           'end_datetime': pd.Timestamp.utcnow().date() - timedelta(days=1),
                                                            'scheduled_job': True, 'freq': 'day'}, 
                   ('20210101-150000', '20210101-160000', True, 'day'): {'start_datetime': pd.Timestamp.utcnow().date() - timedelta(days=1), 
                                                                         'end_datetime': pd.Timestamp.utcnow().date() - timedelta(days=1),
                                                                         'scheduled_job': True, 'freq': 'day'},
                   ('20210101', '20210131', True, 'hour'): {'start_datetime': (pd.Timestamp.utcnow()  - timedelta(hours=1)).replace(minute=0, second=0, microsecond=0),
                                                            'end_datetime': pd.Timestamp.utcnow().replace(minute=0, second=0, microsecond=0),
                                                            'scheduled_job': True, 'freq': 'hour', }, 
                   ('20210101-150000', '20210101-160000', True, 'hour'): {
                                                            'start_datetime': (pd.Timestamp.utcnow() - timedelta(hours=1)).replace(minute=0, second=0, microsecond=0),
                                                            'end_datetime': pd.Timestamp.utcnow().replace(minute=0, second=0, microsecond=0),
                                                            'scheduled_job': True, 'freq': 'hour'}, 
                   ('20210101', '20210131', False, 'day'): {'start_datetime': '2021-01-01 00:00:00', 'end_datetime': '2021-01-31 00:00:00', 'scheduled_job': False, 'freq': 'day'},
                   ('20210101-150000', '20210101-160000', False, 'day'): {'start_datetime': '2021-01-01 15:00:00', 'end_datetime': '2021-01-01 16:00:00', 'scheduled_job': False, 'freq': 'day'}, 
                   ('20210101', '20210131', False, 'hour'): {'start_datetime': '2021-01-01 00:00:00', 'end_datetime': '2021-01-31 00:00:00', 'scheduled_job': False, 'freq': 'hour'}, 
                   ('20210101-150000', '20210101-160000', False, 'hour'): {'start_datetime': '2021-01-01 15:00:00', 'end_datetime': '2021-01-01 16:00:00', 'scheduled_job': False, 'freq': 'hour'}}
    cmdparser = ToolCmdParser()
    for key, value in testing_map.items():
        argString = ""
        argString += f" --start_datetime {key[0]} --end_datetime {key[1]} "
        if key[2]:
            argString += f' --scheduled_job --freq {key[3]} '
        else:
            argString += f' --freq {key[3]} '
        cmdparser.args = cmdparser.parser.parse_args(shlex.split(argString))
        cmdparser.get_args()
        assert pd.to_datetime(cmdparser.start_datetime, utc=True) == pd.to_datetime(value.get('start_datetime'), utc=True), f'{cmdparser.start_datetime} != {value.get("start_datetime")}'
        assert pd.to_datetime(cmdparser.end_datetime, utc=True) == pd.to_datetime(value.get('end_datetime'), utc=True), f'{cmdparser.end_datetime} != {value.get("end_datetime")}'
        assert cmdparser.scheduled_job == value.get('scheduled_job'), f'{cmdparser.scheduled_job} != {value.get("scheduled_job")}'
        assert cmdparser.freq == value.get('freq')
        print(f'{key} passed')




from statsmodels.tsa.stattools import adfuller
import pandas as pd
from kasheesh_tools.tool_logger import get_my_logger

def adf_summary(df, autolag='AIC', pvalue_threshold=0.05):
    logger = get_my_logger(name="adf_summary", level="INFO")
    logger.info('Results of Augmented Dickey-Fuller Test:')
    dftest = adfuller(df, autolag=autolag)
    dfoutput = pd.Series(dftest[0:4], index=['T-Test Statistic','p-value','#Lags Used','Number of Observations Used'])

    for key,value in dftest[4].items():
        dfoutput['Critical Value (%s)'%key] = value

    print(dfoutput)
    if dftest[1] > pvalue_threshold:
        logger.info("Failed to reject H0(the data is non-stationary). This data is non-stationary, cannot use time series prediction")
    else:
        logger.info("Rejecting H0(the data is non-stationary). This data is stationary, can use time series prediction")



"""This is the tool used to parse common-used command line arguments for Kasheesh ETLs.
"""


import argparse
from pandas import to_datetime, Timedelta, Timestamp
from kasheesh_tools.tool_logger import get_my_logger

class ToolCmdParser:
    def __init__(self, name='ToolCmdParser', custom_args=None):
        """* no arguments given: full history
           * only given start_date: data on and after start_date
           * only given end_date: data on and before end_date
           * only given --scheduled_job: data on current date
           * given mixed start_date/end_date & scheduled_job: start_date/end_date will be ignored and only run data on current date

        Args:
            name (str, optional): _description_. Defaults to 'ToolCmdParser'.
            custom_args (list, optional): [(name, type, help)]. Defaults to None.
        """
        self.logger = get_my_logger(name=name)
        self.parser = argparse.ArgumentParser()
        if custom_args:
            for arg in custom_args:
                self.parser.add_argument(f'--{arg[0]}', type=arg[1], help=arg[2])
        self.parser.add_argument('--start_datetime', type=str, help='starting datetime of the transactions')
        self.parser.add_argument('--end_datetime', type=str, help='ending datetime of the transactions')
        self.parser.add_argument('--scheduled_job', type=bool, help='whether this is a scheduled job', action=argparse.BooleanOptionalAction)
        self.parser.add_argument('--freq', type=str, help='frequency of the scheduled job')
        self.args = self.parser.parse_args()

    def _get_freq(self):
        if self.args.freq:
            self.freq = self.args.freq
        else:
            self.freq = 'day'
        self.logger.info(f'freq: {self.freq}')

    def _get_start_datetime(self):
        if self.args.start_datetime:
            if self.args.scheduled_job:
                self.logger.warning('start_datetime will be ignored for scheduled jobs.')
            self.start_datetime = to_datetime(self.args.start_datetime)
        else:
            self.start_datetime = None
        self.logger.info(f'start_datetime: {self.start_datetime}')

    def _get_end_datetime(self):
        if self.args.end_datetime:
            if self.args.scheduled_job:
                self.logger.warning('end_datetime will be ignored for scheduled jobs.')
            self.end_datetime = to_datetime(self.args.end_datetime)
        else:
            self.end_datetime = None
        self.logger.info(f'end_datetime: {self.end_datetime}')

    def _get_scheduled_job(self):
        if self.args.scheduled_job:
            self.scheduled_job = True
            if self.freq == 'day':
                standard_date = Timestamp.utcnow()
                self.start_datetime = standard_date - Timedelta(days=1)
                self.start_datetime = self.start_datetime.replace(hour=0, minute=0, second=0, microsecond=0)
                self.end_datetime = standard_date - Timedelta(days=1)
                self.end_datetime = self.end_datetime.replace(hour=0, minute=0, second=0, microsecond=0)
            elif self.freq == 'hour':
                standard_date = Timestamp.utcnow()
                self.start_datetime = standard_date - Timedelta(hours=1)
                self.start_datetime = self.start_datetime.replace(minute=0, second=0, microsecond=0)
                self.end_datetime = standard_date
                self.end_datetime = self.end_datetime.replace(minute=0, second=0, microsecond=0)
            else:
                self.logger.error('Invalid frequency.')
        else:
            self.scheduled_job = False
        self.logger.info(f'scheduled_job: {self.scheduled_job}')

    def get_args(self):
        self._get_freq()
        self._get_start_datetime()
        self._get_end_datetime()
        self.start_date = self.start_datetime
        self.end_date = self.end_datetime
        self._get_scheduled_job() # this will override the start_date and end_date if it's a scheduled job
        return